<?php

namespace App\Helpers;

use Carbon\Carbon;
use Illuminate\Support\Facades\URL;
use Illuminate\Support\Str;
use Vinkla\Hashids\Facades\Hashids;

class Helper
{
    /**
     * Json response helper
     *
     * @param $data
     * @param int $code
     * @param string[] $messages
     * @param array $extra
     * @return \Illuminate\Http\JsonResponse
     */
    public static function response($data, $code = 200, $pagination = false,  $messages = ["Request Completed Successfully"], $extra = [])
    {
        $response = array_merge([
            "code" => $code,
            "messages" => $messages
        ], $extra);

        $response['data'] = [];

        if ($code >= 200 && $data) {

            if ($pagination) {

                $data = $data->response()->getData(true);

                $response = array_merge([
                    "code" => $code,
                    "messages" => $messages,
                    'data' => $data['data'],
                    'pagination' => $data['pagination']
                ], $extra);
            } else {
                $response['data'] = $data;
            }
        }


        if ($code == 406 || $code == 401 || $code >= 400) {
            $response = [
                "code" => $code,
                "messages" => $messages
            ];
            return response()->json(['error' => $response], $code);
        }

        return response()->json(['response' => $response], $code);
    }

    /*     public static function response($data, $code = 200, $messages = ['Request Completed Successfully'], $extra = [])
    {
        $response = [
            "code" => $code,
            "messages" => $messages
        ];

        $response['data'] = [];

        if ($code >= 200 && $data) {
            if (isset($data['pagination']) && $data['pagination']) {
                $response['data'] = $data['data'];
                $response['pagination'] = $data['pagination'];
            } else {
                $response['data'] = $data;
            }
        }

        if ($code >= 400) {
            return response()->json(['error' => $response], $code);
        } else {
            return response()->json(['response' => array_merge(
                $response,
                $extra
            )], $code);
        }
    } */

    /**
     * Encode id
     *
     * @param $id
     * @return string
     */
    public static function encode($id)
    {
        if (env('HASHIDS') && is_numeric($id)) {
            return Hashids::encode($id);
        }

        if (!env('HASHIDS') && !is_numeric($id)) {
            return Hashids::encode($id);
        }

        return (int) $id;
    }

    /**
     * Decode id
     *
     * @param $str
     * @return int
     */
    public static function decode($str)
    {
        if (env('HASHIDS') && !is_numeric($str)) {
            $decode = Hashids::decode($str);
            return (int) reset($decode);
        }

        if (!env('HASHIDS') && !is_numeric($str)) {
            $decode = Hashids::decode($str);
            return (int) reset($decode);
        }

        return (int) $str;
    }

    /**
     * Decode multiple ids
     *
     * @param $ids
     * @return array|int
     */
    public static function decodeIds($ids)
    {
        $decodedIds = [];

        if (strpos($ids, ',') > 0) {
            $idsArray = explode(',', $ids);

            foreach ($idsArray as $key => $value) {
                $decodedIds[$key] = self::decode($value);
            }

            return $decodedIds;
        }

        return self::decode($ids);
    }

    /**
     * Get uploads directory path
     *
     * @return string
     */
    public static function getUploadsPath()
    {
        return dirname(base_path() . '/') . '/rentals-uploads/';
    }

    /**
     * Get the last segment of commentable model
     *
     * @param $commentable
     * @return mixed
     */
    public static function getCommentableModel($commentable)
    {
        $model = explode('\\', $commentable);

        return strtolower(end($model));
    }

    /**
     * Get temporary signed URL
     *
     * @param $route
     * @param $time
     * @param $payload
     * @return string
     */
    public static function getSignedUrl($route, $time, $payload)
    {
        return URL::temporarySignedRoute($route, $time, $payload);
    }

    /**
     * Get verification code
     *
     * @return mixed
     */
    public static function getVerificationCode()
    {
        return Hashids::connection('verification_code')
            ->encode(Carbon::now()->format('Y-m-d H:i:s'));
    }

    /**
     * Create slug if not exists in request
     *
     * @param $request
     * @return string|null
     */
    public static function getSlug($request)
    {
        if ((!$request->has('slug') && !$request->filled('slug')) || $request->slug == "") {
            return Str::slug($request->title, '-');
        }

        return $request->slug;
    }

    /**
     * for upload media and making media path
     *
     * @param  mixed $request
     * @return void
     */
    public function media($request)
    {
        $extension = $request->file('profile_image')->getClientOriginalExtension();

        return $request->file('profile_image')->storeAs(
            'profile_image/' . Helper::encode(auth()->user()->id),
            Str::random(10) . '.' . $extension,
            env('DISK')
        );
    }
}
