<?php

namespace App\Http\Controllers;

use App\Events\PostActionNotification;
use App\Http\Resources\AppointmentResource;
use App\Listeners\SendActionNotification;
use App\Models\Activity;
use App\Models\Appointment;
use App\Models\Category;
use App\Models\TaxAndCharge;
use App\Models\User;
use App\Traits\HashId;
use App\Traits\Response;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class AppointmentController extends Controller
{
    use Response, HashId;

    /**
     * store
     *
     * @param  mixed $request
     * @return void
     */
    public function store(Request $request)
    {

        $user = auth()->user();
        $input = $request->all();

        $input['user_id'] = $user->id;
        $input['service'] = Category::find($request['service_category_id']);
        $input['verification_code'] = rand(1000, 9999);
        $input['service_status'] = 'pending';

        if ($request->booking_type === 'job') {

            $body = "New job of {$input['service']->title}.";

            $attributes = $this->limitedAttr();

            $data = User::whereNotIn('user_role', ['customer', 'franchise'])
                //->where(['is_online' => 1])
                ->select($attributes);
                $data->excludeBlocked();
            // if ($request->service_category_id && $request->service_category_id != 'null') $data = $data->hasCategory($this->decodeIds(explode(",", $request->service_category_id)));
            // if ($request->preferences && $request->preferences != 'null') $data = $data->hasPreferences($this->decodeIds(explode(",", $request->preferences)));
            // if ($request->language && $request->language != 'null') $data = $data->hasLanguage($request->language);
            // if ($request->qualification && $request->qualification != 'null') $data = $data->hasQualification(explode(",", $request->qualification));
             if ($request->latitude && $request->longitude) {

                $latitude = $request->latitude;
                $longitude = $request->longitude;

                $distance = DB::raw("latitude, longitude, 111.111 *
                DEGREES(ACOS(LEAST(1.0, COS(RADIANS(latitude))
                    * COS(RADIANS( $latitude ))
                    * COS(RADIANS( longitude - $longitude ))
                    + SIN(RADIANS( latitude))
                    * SIN(RADIANS( $latitude )), 1.0))) AS distance_in_km");

                $data = $data
                    ->where('latitude', '!=', null)
                    ->select(array_merge($attributes, [$distance]))
                    ->having('distance_in_km', '<', 161)
                    //->groupBy('users.id')
                    ->orderBy('distance_in_km', 'asc');
            }
            $data = $data->get();
            if ($data->count() <= 0) {
                return $this->response("", 404, false, 'no care giver avialable with your selected criteria');
            }
            $appointment = Appointment::create($input);
            event(new PostActionNotification(
                $data->pluck('id')->toArray(),
                ['App\Models\User'],
                [
                    'title' => $body,
                    'body' => $body,
                    'object_id' => $appointment->id,
                    'object_type' => 'App\Models\Appointment',
                    'actor_id' => $user->id,
                    'actor_type' => 'App\Models\User',
                    'type' => 'job_posted',
                    'object' => $body,

                ],
                null,
                true
            ));
            //$nearby = User::nearBy($appointment->latitude, $appointment->longitude, 5)->get();
        } elseif ($request->booking_type == "selection") {

            $body = "$user->full_name has requested a booking of {$input['service']->title} on {$request->start_date} {$request->start_time}.";

            if ($request->appointment_id && $request->appointment_id != 'null')
            {
                $previousbooking = Appointment::find($input['appointment_id']);
                $input=$previousbooking->toArray();
                //$appointment = Appointment::create($input);
                $previousbooking->delete();
            }
            $input['booking_type'] = 'selection';
            $appointment = Appointment::create($input);
            event(new PostActionNotification(
                [$request->service_provider_id],
                ['App\Models\User'],
                [
                    'title' => $body,
                    'body' => $body,
                    'object_id' => $appointment->id,
                    'object_type' => Appointment::class,
                    'actor_id' => $user->id,
                    'actor_type' => User::class,
                    'type' => 'appointment_request',
                    'object' => $body,

                ],
                null,
                true
            ));
        } elseif ($request->booking_type === 'instant') {

            $body = "New job of {$input['service']->title}.";

            $attributes = $this->limitedAttr();

            $data = User::whereNotIn('user_role', ['customer', 'franchise'])
                //->where(['is_online' => 1])
                ->select($attributes);
                $data->excludeBlocked();
            // if ($request->category && $request->category != 'null') $data = $data->hasCategory($this->decodeIds(explode(",", $request->category)));
            // if ($request->preferences && $request->preferences != 'null') $data = $data->hasPreferences($this->decodeIds(explode(",", $request->preferences)));
            // if ($request->language && $request->language != 'null') $data = $data->hasLanguage($request->language);
            // if ($request->qualification && $request->qualification != 'null') $data = $data->hasQualification(explode(",", $request->qualification));
            if ($request->latitude && $request->longitude) {

                $latitude = $request->latitude;
                $longitude = $request->longitude;

                $distance = DB::raw("latitude, longitude, 111.111 *
                DEGREES(ACOS(LEAST(1.0, COS(RADIANS(latitude))
                    * COS(RADIANS( $latitude ))
                    * COS(RADIANS( longitude - $longitude ))
                    + SIN(RADIANS( latitude))
                    * SIN(RADIANS( $latitude )), 1.0))) AS distance_in_km");

                $data = $data
                    ->where('latitude', '!=', null)
                    ->select(array_merge($attributes, [$distance]))
                    ->having('distance_in_km', '<', 161)
                    //->groupBy('users.id')
                    ->orderBy('distance_in_km', 'asc');
            }
            $data = $data->get();
            //dd($data->count());
            if ($data->count() <= 0) {
                return $this->response("", 404, false, 'no care giver avialable with your selected criteria');
            }
            $appointment = Appointment::create($input);

            event(new PostActionNotification(
                $data->pluck('id')->toArray(),
                ['App\Models\User'],
                [
                    'title' => $body,
                    'body' => $body,
                    'object_id' => $appointment->id,
                    'object_type' => 'App\Models\Appointment',
                    'actor_id' => $user->id,
                    'actor_type' => 'App\Models\User',
                    'type' => 'instant_request',
                    'object' => $body,

                ],
                null,
                true
            ));

            //$nearby = User::nearBy($appointment->latitude, $appointment->longitude, 5)->get();

        }

        return $this->response($appointment, 200, false, 'Job posted successfully');
    }

    /**
     * acceptService
     *
     * @param  mixed $request
     * @return void
     */
    public function acceptService(Request $request)
    {
        $input = $request->all();
        $order = Appointment::find($this->decode($input['appointment_id']));
        $order->update([
            'service_provider_id' => auth()->user()->id,
            'service_status' => 'accepted',
            'rate_per_hour' => auth()->user()->service_charge,
            'pay_range' => auth()->user()->service_charge * $order->duration,
        ]);
        /*   $order->bookingDetails()->update([
            'service_status' => 'accepted',
        ]); */
        /*  $service_provider_rating = User::withCount(['rating as average_rating' => function ($query) {
            $query->select(\DB::raw('coalesce(avg(rating),0)'));
        }])->where('id', auth()->user()->id)->first(); */

        event(new PostActionNotification(
            [$order->user_id],
            ['App\Models\User'],
            [
                'title' => 'Service accepted',
                'body' => 'Order accepted by ' . auth()->user()->full_name,
                'object_id' => $order->id,
                'actor_id' => auth()->user()->id,
                'actor_type' => 'App\Models\User',
                'type' => 'order_accept',
                'object' => 'find service provider',
                'details' => [
                    'order_id' => $this->encode($order->id),
                    'pickup_address' => $order->pickup_location,
                    'customer_name' => auth()->user()->full_name,
                    'dropoff_address' => $order->dropoff_location,
                    'customer_image' => auth()->user()->profile_image,
                    //'rating' => $service_provider_rating,
                ],
            ],
            null,
            true
        ));

        $activity = Activity::where([
            'user_id' => auth()->user()->id,
            'type' => 'appointment_request',
            'object_id' => $order->id

        ])->first();
        if($activity)
        {
            $activity->delete();
        }




        return $this->response(['booking' => $order], 200, false, ['Order accepted successfully']);
    }

    /**
     * rejectService
     *
     * @param  mixed $id
     * @return void
     */
    public function rejectService(Request $request)
    {
        $inputnanny = $request->all();
        $favnanny = Appointment::where([
            'id' => $this->decode($inputnanny['appointment_id']),
            'service_provider_id' => auth()->user()->id
        ])->first();
        $favnanny->service_status = 'rejected';
        $favnanny->save();

        event(new PostActionNotification(
            [$favnanny->user_id],
            ['App\Models\User'],
            [
                'title' => 'Service rejected',
                'body' => 'Service rejected ' . $favnanny->service_provider->full_name,
                'object_id' => $favnanny->id,
                'actor_id' => $favnanny->service_provider_id,
                'actor_type' => 'App\Models\User',
                'type' => 'service_rejected',
                'object' => 'service rejected',
                'details' => [
                    'order_id' => $favnanny->id,
                    'pickup_address' => $favnanny->pickup_location,
                    'nanny_name' => $favnanny->service_provider->full_name,
                    'dropoff_address' => $favnanny->dropoff_location,
                    'description' => $favnanny->description,
                    'customer_image' => $favnanny->customer->profile_image,

                ],
            ],
            null,
            true
        ));

        $activity = Activity::where([
            'user_id' => auth()->user()->id,
            'type' => 'appointment_request',
            'object_id' => $favnanny->id

        ])->first();
        if($activity)
        {
            $activity->delete();
        }

        return $this->response([], 200, false, ["Service rejected successfully"]);
    }


    /**
     * arrived
     *
     * @param  mixed $request
     * @return void
     */
    public function arrived(Request $request)
    {
        $input = $request->all();
        $id = $this->decode($input['appointment_id']);
        $order = Appointment::where('service_provider_id', auth()->user()->id)
           // ->where('service_status', 'accepted')
            ->where('id', $id)
            ->first();

        if (!$order) {
            return $this->response("", 404, false, ['No order found']);
        }

        $order->update([
            'service_status' => 'arrived',
        ]);

        // send notification to customer
        event(new PostActionNotification(
            [$order->customer->id],
            ['App\Models\User'],
            [
                'title' => 'Service provider arrived',
                'body' => 'Service provider arrived at your location',
                'object_id' => $order->id,
                'actor_id' => auth()->user()->id,
                'actor_type' => 'App\Models\User',
                'type' => 'service_provider_arrived',
                'object' => 'find service provider',
                'details' => [
                    'order_id' => $this->encode($order->id),
                    'pickup_address' => $order->pickup_location,
                    'nanny_name' => auth()->user()->full_name,
                    'dropoff_address' => $order->dropoff_location,
                    'nanny_image' => auth()->user()->profile_image,
                ],
            ],
            null,
            true
        ));
        return $this->response(['booking' => $order], 200, false, ['Nanny arrived']);
    }


    /**
     * startService
     *
     * @param  mixed $request
     * @return void
     */
    public function startService(Request $request)
    {

        $input = $request->all();
        $id = $this->decode($input['appointment_id']);
        $order = Appointment::where('service_provider_id', auth()->user()->id)
            //->where('service_status', 'arrived')
            ->where('id', $id)
            ->first();
        if (!$order) {
            return $this->response("", 404, false, ['No order found']);
        }
        if ($order->verification_code != $input['verification_code']) {
            return $this->response("", 404, false, ['Verification code is incorrect']);
        }

        $order->update([
            'service_status' => 'started',
            'started_at' => Carbon::now(),
            'start_time' => Carbon::now(),
        ]);

        // send notification to customer
        event(new PostActionNotification(
            [$order->user_id],
            ['App\Models\User'],
            [
                'title' => 'Service started',
                'body' => 'Service started',
                'object_id' => $order->id,
                'actor_id' => auth()->user()->id,
                'actor_type' => 'App\Models\User',
                'type' => 'service_started',
                'object' => 'find service provider',
                'details' => [
                    'order_id' => $this->encode($order->id),
                    'pickup_address' => $order->pickup_location,
                    'nanny_name' => auth()->user()->full_name,
                    'dropoff_address' => $order->dropoff_location,
                    'nanny_image' => auth()->user()->profile_image,
                ],
            ],
            null,
            true
        ));
        return $this->response(['booking' => $order], 200, false, ['Service started']);
    }

    /**
     * endService
     *
     * @param  mixed $request
     * @return void
     */

    public function endService(Request $request)
    {
        $input = $request->all();
        $id = $this->decode($input['appointment_id']);
        $order = Appointment::where('service_provider_id', auth()->user()->id)
            //->whereIn('service_status', ['approved-receiver','rejected-receiver'])
            ->where('id', $id)
            ->first();
        if (!$order) {
            return $this->response("", 404, false, ['No order found']);
        }

        if(Carbon::parse($order->started_at)->addHours($order->duration) > Carbon::now() ) //2023-05-03 07:12:29
        {

        //return $this->response('', 422, false, ['Service Time Not Completed ']);


        }
        $startDateTime = Carbon::parse($order->started_at);

        $order->update([
            'service_status' => 'completed',
            'end_time' => Carbon::now(),
             'working_hours' => $startDateTime->diffInHours(Carbon::now()),


        ]);

        // send notification to customer
        event(new PostActionNotification(
            [$order->customer->id],
            ['App\Models\User'],
            [
                'title' => 'Service completed',
                'body' => 'Service completed',
                'object_id' => $order->id,
                'actor_id' => auth()->user()->id,
                'actor_type' => 'App\Models\User',
                'type' => 'service_completed',
                'object' => 'find service provider',
                'details' => [
                    'order_id' => $this->encode($order->id),
                    'pickup_address' => $order->pickup_location,
                    'nanny_name' => auth()->user()->full_name,
                    'dropoff_address' => $order->dropoff_location,
                    'nanny_image' => auth()->user()->profile_image,
                ],
            ],
            null,
            true
        ));
        // $order->charge($order->transaction_reference, url('invoice/callback'), User::find($order->user_id));
        // if ($order->redirect_url != null) {
        //     return $this->response($order, 200, false, 'Payment action required');
        // }
        auth()->user()->wallet->deposit($order->duration * auth()->user()->service_charge, "payment");
        return $this->response(['booking' => $order], 200, false, ['Service completed']);
    }

    /**
     * cancelService
     *
     * @param  mixed $request
     * @return void
     */
    public function cancelService(Request $request)
    {
        $input = $request->all();
        $id = $this->decode($input['appointment_id']);
        $booking = Appointment::find($id);
        if (!$booking) {
            return $this->response("", 404, false, ['No booking found']);
        }

        $status = auth()->user()->user_type == 'nanny' ? 'cancelled-by-nanny' : 'cancelled-by-user';

        $tax_and_charges = TaxAndCharge::where('charged_for', auth()->user()->user_type)->get();

        $pelanty_amount = [];
        foreach ($tax_and_charges as $pelanty) {
            if ($pelanty->operation_type == 'amount' && $pelanty->charge_type == 'penalty') {
                $pelanty_amount[] = $pelanty->value;
            }
        }
        if (auth()->user()->user_type == 'customer') {
            $update_check =   $booking->update([
                'service_status' => $status,
                'appointment_cancel_time' => Carbon::now(),
            ]);
            event(new PostActionNotification(
                [$booking->nanny->id],
                ['App\Models\User'],
                [
                    'title' => 'Service cancelled',
                    'body' => 'Service cancelled',
                    'object_id' => $booking->id,
                    'actor_id' => auth()->user()->id,
                    'actor_type' => 'App\Models\User',
                    'type' => 'service_cancelled',
                    'object' => 'service cancelled',
                    'details' => [
                        'order_id' => $this->encode($booking->id),
                    ],
                ],
                null,
                true
            ));
            $booking->charge($booking->transaction_reference, url('invoice/callback'));
            if ($booking->redirect_url != null) {
                return $this->response($booking, 200, false, 'Payment action required');
            }

            return $this->response(['booking' => new AppointmentResource($booking)], 200, false, ['Order cancelled successfully']);
        }

        auth()->user()->wallet->penalty(array_sum($pelanty_amount), "penalty");

        $booking->update([
            'service_status' => $status,
            'cancel_reason' => $input['cancel_reason'],
        ]);

        event(new PostActionNotification(
            [$booking->user->id],
            ['App\Models\User'],
            [
                'title' => 'Service cancelled',
                'body' => 'Service cancelled ' . auth()->user()->user_type,
                'object_id' => $booking->id,
                'actor_id' => auth()->user()->id,
                'actor_type' => 'App\Models\User',
                'type' => 'service_cancelled',
                'object' => 'service cancelled',
                'details' => [
                    'order_id' => $this->encode($booking->id),
                ],
            ],
            null,
            true
        ));


        return $this->response(['booking' => new AppointmentResource($booking)], 200, false, ['Order cancelled successfully']);
    }


    public function acknowledge(Request $request)
    {

        $input = $request->all();
        $id = $this->decode($input['appointment_id']);
        $order = Appointment::where('user_id', auth()->user()->id)
            ->where('service_status', 'completed')
            ->where('id', $id)
            ->first();
        if (!$order) {
            return $this->response("", 404, false, ['No order found']);
        }

        $order->update([
            'service_status' => 'acknowledge',
            'started_at' => Carbon::now(),
        ]);

        // send notification to customer
        // event(new PostActionNotification(
        //     [$order->service_provider_id],
        //     ['App\Models\User'],
        //     [
        //         'title' => 'Appointment acknowledge',
        //         'body' => 'Appointment acknowledge',
        //         'object_id' => $order->id,
        //         'actor_id' => auth()->user()->id,
        //         'actor_type' => 'App\Models\User',
        //         'type' => 'appointment_acknowledge',
        //         'object' => 'Appointment acknowledge',
        //         'details' => [
        //             'order_id' => $this->encode($order->id),
        //             'pickup_address' => $order->pickup_location,
        //             'nanny_name' => auth()->user()->full_name,
        //             'dropoff_address' => $order->dropoff_location,
        //             'nanny_image' => auth()->user()->profile_image,
        //         ],
        //     ],
        //     null,
        //     true
        // ));
        return $this->response(['booking' => $order], 200, false, ['Service acknowledge']);
    }

    public function extend(Request $request)
    {
        $user = auth()->user();
        $appointment = Appointment::where('id', $request->parent_id)->first()->toArray();
        $input =  $appointment;
        $input['parent_id'] = $request->parent_id;
        $input['duration'] = $request->duration;
        $input['service_status'] = 'pending';

        $body = "{$user->full_name} has requested for {$request->duration} hours of time extension";
        $new_appointment = Appointment::create($input);

        event(new PostActionNotification(
            [$input['service_provider_id']],
            ['App\Models\User'],
            [
                'title' => $body,
                'body' => $body,
                'object_id' => $new_appointment->id,
                'object_type' => Appointment::class,
                'actor_id' => auth()->user()->id,
                'actor_type' => User::class,
                'type' => 'appointment_extension',
                'object' => $body,

            ],
            null,
            true
        ));

        return $this->response($new_appointment, 200, false, ['Request for time extension has been sent successfully']);
    }


    public function appointments(Request $request)
    {
        $appointment = new Appointment();
        if (auth()->user()->user_role == 'customer') {
            $appointment = $appointment->where('user_id', auth()->user()->id);
        } else {
            $appointment = $appointment->where('service_provider_id', auth()->user()->id);
        }
        $appointment = $appointment->whereIn('booking_type', ['selection', 'instant']);
        if ($request->status == 'schedule') {
            $appointment = $appointment->where('start_date', '>=', Carbon::now()->format('Y-m-d'))
            ->whereIn('service_status', ['pending','accepted']);
        } elseif ($request->status == 'in_progress') {
            $appointment = $appointment->where('start_date', '<=', Carbon::now()->format('Y-m-d'))
            ->where('service_provider_id' ,'!=', null)
            ->whereIn('service_status', ['pending','accepted', 'arrived', 'started']);
        } elseif ($request->status == 'completed') {
            $appointment = $appointment->whereIn('service_status', ['completed', 'cancelled-by-nanny', 'cancelled-by-user','acknowledge']);
        }
        $appointment = $appointment->get();
        \Log::info($appointment);

        return $this->response(AppointmentResource::collection($appointment), 200, false);
    }

    public function appointment($id)
    {
        $appointment = Appointment::find($this->decode($id));

        return $this->response(new AppointmentResource($appointment), 200, false);
    }

    /**
     * limitedAttr
     *
     * @return void
     */
    protected function limitedAttr()
    {
        return [
            'users.id',
            'users.full_name',
            'users.email',
            'users.age',
            'users.profile_image',
            'users.is_online',
            'users.user_role',
        ];
    }
}
