<?php

namespace App\Http\Controllers;

use App\Helpers\Helper;
use App\Http\Requests\ChangePasswordRequest;
use App\Models\User;
use Illuminate\Support\Facades\Hash;

class ChangePasswordController extends Controller
{
    /**
     * Change password
     *
     * @param ChangePasswordRequest $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function changePassword(ChangePasswordRequest $request)
    {
        $attributes = $this->attributes($request);

        // If current password is mismatched
        if (!Hash::check($attributes['current_password'], auth()->user()->password)) {
            return Helper::response([], 422, false, ['Your current password is incorrect']);
        }

        // If current password is same as new password
        if (Hash::check($attributes['password'], auth()->user()->password)) {
            return Helper::response([], 422, false, [
                'New Password must be different from the Current Password'
            ]);
        }

        User::find(auth()->user()->id)->update(['password' => $attributes['password']]);

        return Helper::response(["user" => auth()->user()], 200, false, ['Your password has been reset successfully']);
    }

    /**
     * Change password attributes
     *
     * @param $request
     * @return mixed
     */
    private function attributes($request)
    {
        return $request->only([
            'current_password',
            'password'
        ]);
    }
}
