<?php

namespace App\Http\Controllers;

use App\Http\Resources\AppliedJobResource;
use App\Http\Resources\PaginationCollection;
use App\Http\Resources\UserResource;
use App\Models\Appointment;
use App\Models\Category;
use App\Models\User;
use App\Traits\HashId;
use App\Traits\Response;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class HomeController extends Controller
{
    use HashId, Response;

    public function all(Request $request)
    {

        $attributes = $this->limitedAttr();
        $auth = $request->user();
        $age_range = explode(",", $request->age);
        //$from = Carbon::parse(now()->sub($age_range[0] . ' years')->calendar())->format('Y-m-d');
        //$to = Carbon::parse(now()->sub($age_range[1] . ' years')->calendar())->format('Y-m-d');

        $data = User::
            //where( $request->only('user_role', 'category') )
            whereNotIn('user_role', ['customer', 'franchise'])
            ->where(['is_online' => 1])
            ->select($attributes);
        $data->excludeBlocked();

        if ($request->favorite) {
            $data = $data->join('user_favorites as uf', 'uf.favorite_id', '=', 'users.id')->where('uf.user_id', $auth->id);
        }
        //if ($request->asap) $data = $data->needAsap();
        // if ($request->age && $request->age != 'null') $data = $data->hasAge($age_range);
        if ($request->search && $request->search != 'null') $data = $data->hasSearch(str_replace("-", " ", $request->search));
        if ($request->category && $request->category != 'null') $data = $data->hasCategory($this->decodeIds(explode(",", $request->category)));
        if ($request->preferences && $request->preferences != 'null') $data = $data->hasPreferences($this->decodeIds(explode(",", $request->preferences)));
        if ($request->languages && $request->languages != 'null') $data = $data->hasLanguage($request->languages);
        if ($request->qualification && $request->qualification != 'null') $data = $data->hasQualification(str_replace("-", " ", $request->qualification));
        //if( $request->rating ) $data = $data->hasRating( $request->rating );


        if ($request->latitude && $request->longitude) {

            $latitude = $request->latitude;
            $longitude = $request->longitude;

            $distance = DB::raw("latitude, longitude, 111.111 *
            DEGREES(ACOS(LEAST(1.0, COS(RADIANS(latitude))
                * COS(RADIANS( $latitude ))
                * COS(RADIANS( longitude - $longitude ))
                + SIN(RADIANS( latitude))
                * SIN(RADIANS( $latitude )), 1.0))) AS distance_in_km");

            $data = $data
                ->where('latitude', '!=', null)
                ->select(array_merge($attributes, [$distance]))
                ->having('distance_in_km', '<', $request->radius)
                //->groupBy('users.id')
                ->orderBy('distance_in_km', 'asc');
        }
        $data = $data->get();
        // dd($data);
        if (count($data) < 0) {
            return $this->response('', 404, false, ['No care giver found']);
        }
        return $this->response(['care_givers' => UserResource::collection($data)], 200, false, []);
    }

    public function jobs(Request $request)
    {

        $appointments = Appointment::with([
            'customer', 'category'
        ])
            //->whereNotIn('status', ['hired'] )
            ->join('activities as a', function ($q) {
                $q->on('a.object_id', '=', 'appointments.id')
                    ->where('a.type', 'job_posted')
                    ->where('a.deleted_at', NULL)
                    ->where("a.user_id", auth()->user()->id);
            })
            //->where('a.action', 'job_posted')
            ->where('booking_type', 'job')
            ->orderBy('appointments.id', 'desc')
            ->select('appointments.*')
            ->get();

        ///
        $instant_requests = Appointment::with([
            'customer', 'category'
        ])
            //->whereNotIn('status', ['hired'] )
            ->join('activities as a', function ($q) {
                $q->on('a.object_id', '=', 'appointments.id')
                    ->where('a.deleted_at', NULL)
                    ->whereIn('a.type', ['appointment_request', 'instant_request']);
            })
            ->where("a.user_id", auth()->user()->id)
            //->where('a.action', 'job_posted')
            ->whereIn('booking_type', ['selection', 'instant'])
            ->orderBy('appointments.id', 'desc')
            ->select('appointments.*')
            ->get();

        ///

        $applied = Appointment::with([
            'customer', 'category'
        ])

            //->whereNotIn('status', ['hired'] )
            ->whereHas('appliers', function ($q) {
                $q->where('user_id', auth()->user()->id);
                $q->whereIn('status', ['accepted','pending','rejected']);
            })
            ->where('booking_type', 'job')
            ->orderBy('appointments.id', 'desc')
            ->select('appointments.*')
            ->get();
        return $this->response(
            [
                'instant_requests' => $instant_requests,
                'available_jobs' => $appointments,
                'applied_jobs' => AppliedJobResource::collection($applied)
            ]
        );
    }

    protected function limitedAttr()
    {
        return [
            'users.id',
            'users.full_name',
            'users.email',
            'users.age',
            'users.profile_image',
            'users.is_online',
            'users.user_role',
        ];
    }
}
