<?php

namespace App\Http\Controllers;

use App\Events\PostActionNotification;
use App\Http\Resources\AppointmentResource;
use App\Http\Resources\UserResource;
use App\Listeners\SendActionNotification;
use App\Models\Activity;
use App\Models\Appointment;
use App\Models\JobApplier;
use App\Traits\HashId;
use App\Traits\Response;
use Illuminate\Http\Client\Request as ClientRequest;
use Illuminate\Http\Request;

class JobController extends Controller
{
    use HashId, Response;
    public function apply(Request $request)
    {
        $appointment = Appointment::find( $this->decode($request->appointment_id) );
        $user = auth()->user();
        $userExist = $appointment->appliers()->where('user_id', $request->user()->id)->first();
        if( !$userExist ) {
            $appointment->appliers()->attach( $request->user()->id,
            [
                'status' => 'pending',
                'latitude'=> $request->latitude,
                'longitude'=> $request->longitude,
                'address' => $request->address

                ]
        );

            $body = "$user->name applied on your posted job ".$appointment->id;

            event(new PostActionNotification(
                [ $appointment->user_id],
                ['App\Models\User'],
                [
                'title' => $body,
                'body' => $body,
                'object_id' =>$appointment->id,
                'object_type' => 'App\Models\Appointment',
                'actor_id' => $user->id,
                'actor_type' => 'App\Models\User',
                'type' => 'user_applied',
                'object' => $body,

            ],null, true));
            Activity::where([
                'user_id' => auth()->user()->id,
                'type' => 'job_posted',
                'object_id' => $request->appointment_id

            ])->first()->delete();

        }

        return $this->response( null, 200, ['Applied Successfully'] );
    }

    public function removeApply($id, Request $request)
    {
        $appointment = Appointment::find( $this->decode($id) );
        $appointment->appliers()->detach( $request->user()->id );

        return $this->response( null, 200, ['Cancelled Successfully'] );
    }

    public function hireApplier(Request $request)
    {
        $appointment = Appointment::with([
            'appliers',
            'service_provider',
            'customer',
        ])->findOrFail( $request->id );
        $invoice['appointment'] = tap( $appointment )->update([
            "service_provider_id" => $request->service_provider_id,
            "status" => 'accepted',
        ])->toArray();
        $apply = JobApplier::where(['appointment_id' => $appointment->id,
        'user_id' => $request->service_provider_id ])->first();
        $apply->status = "accepted";
        $apply->save();


        $body = "You have been hired on applied job ".$appointment->booking_id;

        event(new PostActionNotification(
            [ $request->service_provider_id],
            ['App\Models\User'],
            [
            'title' => $body,
            'body' => $body,
            'object_id' =>$appointment->id,
            'object_type' => 'App\Models\Appointment',
            'actor_id' => $appointment->user_id,
            'actor_type' => 'App\Models\User',
            'type' => 'user_hired',
            'object' => $body,

        ],null, true));


        return $this->response( $appointment, 200, false, ['Hired Successfully'] );
    }


    public function rejectApplier(Request $request)
    {
        $appointment = Appointment::with([
            'appliers',
            'service_provider',
            'customer',
        ])->findOrFail( $request->id );

        $apply = JobApplier::where(['appointment_id' => $appointment->id,
        'user_id' => $request->service_provider_id ])->first();

        $apply->status = "rejected";
        $apply->save();
        // $apply1 = JobApplier::where(['appointment_id' => $appointment->id,
        // 'user_id' => $request->service_provider_id ])->first();
        //$apply1->delete();



        return $this->response('', 200, false, ['Rejected Successfully'] );
    }



    public function appliers($id)
    {
        $appointment = Appointment::find( $this->decode($id) );
        return $this->response( UserResource::collection($appointment->appliers()->wherePivot('status', '!=', 'rejected')->get()), 200, false, ['Applied care givers'] );
    }

    public function remove($id)
    {
        $appointment = Appointment::findOrFail( $this->decode($id) );
        $appointment->delete();
        return $this->response( null, 200, false, ['Delete Successfully'] );
    }

    public function jobs(Request $request){

        $jobs = new Appointment();
        $jobs = $jobs->where('user_id', auth()->user()->id)->where('booking_type','job');
        if($request->status == 'pending')
        {
        $jobs = $jobs->where('service_provider_id', null);
        }
        else{
        $jobs = $jobs->whereNotNull('service_provider_id')->with('service_provider');
        }
        return $this->response(AppointmentResource::collection($jobs->get()), 200, false);
    }

    public function show($id)
    {
        $job = Appointment::with('service_provider')->findOrFail( $this->decode($id) );

        return $this->response(new AppointmentResource($job) , 200, false);

    }

    public function interested(Request $request)
    {
        $appointment = Appointment::find( $this->decode($request->appointment_id) );
        $user = auth()->user();
        $userExist = $appointment->appliers()->where('user_id', $request->user()->id)->first();
        if( !$userExist ) {
            $appointment->appliers()->attach( $request->user()->id,
            [
                'status' => 'pending',
                'latitude'=> $request->latitude,
                'longitude'=> $request->longitude,
                'address' => $request->address

                ]
        );
        }

            $body = "$user->full_name applied on your posted job ".$appointment->booking_id;

            event(new PostActionNotification(
                [ $appointment->user_id],
                ['App\Models\User'],
                [
                'title' => $body,
                'body' => $body,
                'object_id' =>$appointment->id,
                'object_type' => 'App\Models\Appointment',
                'actor_id' => $user->id,
                'actor_type' => 'App\Models\User',
                'type' => 'user_interested',
                'object' => $body,
                  'details' => [
                    'appointment_id' => $appointment->id,
                ],

            ],null, true));
            $activity = Activity::where([
                'user_id' => auth()->user()->id,
                'type' => 'instant_request',
                'object_id' => $appointment->id

            ])->first();
            if($activity)
            {
                $activity->delete();
            }


        return $this->response("", 200, null, ['Applied Successfully'] );

    }

    public function notInterested(Request $request)
    {
        $activity = Activity::where([
            'user_id' => auth()->user()->id,
            'type' => 'instant_request',
            'object_id' => $request->appointment_id

        ])->first();
        if($activity)
        {
            $activity->delete();
        }

        return $this->response( null, 200, ['Rejected Successfully'] );

    }
}
