<?php

namespace App\Http\Controllers;

use App\Helpers\Helper;
use App\Http\Requests\ForgotPasswordRequest;
use App\Http\Requests\PasswordResetRequest;
use App\Http\Resources\UserResource;
use App\Jobs\ProcessVerification;
use App\Models\User;
use Illuminate\Auth\Events\PasswordReset;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\Password;
use Symfony\Component\Console\Input\Input;

class PasswordResetController extends Controller
{
    // send code to email
    public function forgotPassword(Request $request)
    {
        $input = $request->all();
        if ($request->has('email')) {
            $user = User::where('email', $input['email'])->first();
            if ($user) {
                $user->forgot_password_code = 9999; //rand(1000, 9999);
                $user->save();
                Mail::send('mail.code', ['user' => $user], function ($m) use ($user) {
                    $m->from('donotreply@nannies-on-wheels.com') //from email
                        ->to($user->email) //to email
                        ->subject('Password reset pins'); //email subject
                });
                return Helper::response("", 200, false, ["Password reset pin sent to your email"]);
            } else {
                return Helper::response("", 404, false, ["User not found"]);
            }
        }

        if ($request->has('phone')) {
            $user = User::where('phone', $input['phone'])->first();
            if ($user) {
                $user->forgot_password_code = 9999; //rand(1000, 9999);
                $user->save();
                ProcessVerification::dispatch($user);
                return Helper::response("", 200, false, ["Password reset pin sent to phone"]);
            } else {
                return Helper::response("", 404, false, ["User not found"]);
            }
        }
    }

    public function verifyCode(Request $request)
    {
        $input = $request->all();
        if ($request->has('email')) {
            $user = User::where('email', $input['email'])->first();
        }

        if ($request->has('phone')) {
            $user = User::where('phone', $input['phone'])->first();
            ProcessVerification::dispatch($user);
        }

        if (!$user) {
            return Helper::response("", 404, false, ["User not found"]);
        }

        if ($user->forgot_password_code != $input['code']) {
            return Helper::response("", 404, false, ["Code is not valid"]);
        }
        if (Auth::login($user)) {
            return Helper::response([], 401, false, [__('Email or password is invalid')]);
        }

        $access_token = auth()->user()->createToken('API Token')->plainTextToken;
        $user->forgot_password_code = null;
        $user->save();
        return Helper::response(["user" => new UserResource(auth()->user(), $access_token)], 200);
    }

    public function changePassword(Request $request)
    {
        $user = auth()->user();
        $user->password = $request->password;
        $user->save();
        return Helper::response("", 200, false, ["Password changed successfully"]);
    }
}
