<?php

namespace App\Http\Controllers;

use App\Helpers\Helper;
use App\Http\Requests\UserRegistrationRequest;
use App\Http\Resources\UserResource;
use App\Jobs\ProcessSignUp;
use App\Jobs\ProcessVerification;
use App\Models\User;
use Illuminate\Support\Str;
use Spatie\Activitylog\Contracts\Activity;
use Spatie\Activitylog\Models\Activity as ModelsActivity;

class RegistrationController extends Controller
{
    /**
     * User sign up
     *
     * @param RegistrationRequest $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function register(UserRegistrationRequest $request)
    {
        ProcessSignUp::dispatch(
            $user = User::create($this->attributes($request))
        );
        $log = auth()->login($user);
        $access_token = $user->createToken('API Token')->plainTextToken;

        if ($request->has('udid')) {
            auth()->user()->userDevice($request, $access_token);
        }
        //ProcessVerification::dispatch(auth()->user());
        return Helper::response(["user" => new UserResource($user->fresh(), $access_token)], 201);
    }

    /**
     * Sign up attributes
     *
     * @param $request
     * @return mixed
     */
    private function attributes($request)
    {
        return $request->only([
            'full_name',
            'email',
            'user_role',
            'user_type',
            'phone',
            'password',
            'udid',
            'device_type',
            'device_token',
            'device_brand',
            'device_model',
            'device_os',
            'referred_by',
        ]);
    }
}
