<?php

namespace App\Http\Controllers;

use App\Models\Appointment;
use App\Models\User;
use App\Traits\HashId;
use App\Traits\Response;
use Illuminate\Http\Request;

class TimeSlotsController extends Controller
{
    use HashId, Response ;

    public function all(Request $request)
    {
        $provider = User::find( $this->decode($request->service_provider_id) );

        if( $provider ){

            $output = [];
            foreach ($request->date as $key => $date) {
                $output[$date] = (array) $this->timeSlots( $provider, $date, $request->duration );
            }

            return $this->response($output, 200, false, ['Record found successfully.'] );
        }
        return $this->response("", 404, false, ['User Not Found'] );
    }

    protected function timeSlots($provider, $date, $duration = 0){

        $day = date('l', strtotime( $date ) );
        $schedule = $provider->schedule()
            ->where('status', 'on')
            ->where('day', strtolower($day) )
            ->first();

        $slots = [];
        if( $schedule ){
            // -- Subtract duration for day end
            $end_time = date('H:i:00', strtotime("-$duration hours", strtotime( $schedule['endtime'] ) ) );
            $slots = $this->timeGenerator( $schedule['starttime'], $end_time );
        }

        $appointments = Appointment::where('service_provider_id', $provider->id )
        ->where('date', $date)
        ->whereIn('status', ['pending', 'in_progress', 'completed', 'acknowledged'])
        ->select(['id', 'date', 'start_time', 'end_time', 'time_duration', 'status'])
        ->get();

        foreach( $appointments as $v ) {
            // -- Subtract duration for end time overlapping
            $start_time = date('H:i:00', strtotime("-$duration hours", strtotime( $v->start_time ) ) );

            $reserve = $this->timeGenerator( $start_time, $v->end_time );
            $slots = array_values( array_diff( $slots, $reserve ) );
        }

        return $slots;
    }

    protected function timeGenerator($start, $end, $duration = 30){

        $slots = [];
        while ( $start <= $end ) {
            $slots[] = date('H:i:00', strtotime( $start ) );
            $start = date('H:i', strtotime( $start . "+ $duration minutes" ) );
        }

        return $slots;
    }

    public function allSlots(Request $request){

        // $day = date('l', strtotime( $date ) );
        $user = User::find( $this->decode( $request->service_provider_id ) );
        $schedule = $user->schedule()
            ->select( \DB::raw(' MIN(starttime) as mintime, MAX(endtime) as maxtime ') )
            ->where('status', 'on')
            ->first();
        $slots = [];


        if( $schedule ){
            $time = $schedule['mintime'];
            while ( $time <= $schedule['maxtime'] ) {
                $slots[] = date('H:i', strtotime( $time ) );
                $time = date('H:i', strtotime( $time . ' + 30 minutes' ) );
            }
        }

        return $this->response($slots, 200);
    }
}

