<?php

namespace App\Http\Controllers;

use App\Helpers\Helper;
use App\Events\PostActionNotification;
use App\Http\Requests\ProfileUpdateRequest;
use App\Http\Requests\UserUpdateRequest;
use App\Http\Resources\UserResource;
use App\Jobs\ProcessVerification;
use App\Models\User;
use App\Models\UserCategory;
use App\Traits\HashId;
use App\Traits\Response;
use Illuminate\Http\Request;
use Illuminate\Support\Str;
use App\Services\Checkr;

use Lyal\Checkr\Client;


class UserController extends Controller
{
    use HashId, Response;
    /**
     * Create or update user profile
     *
     * @param UserProfileRequest $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function createOrUpdateProfile(Request $request)
    {
        $attributes = $request->all();
        if (auth()->user()->user_role == 'nanny') {
            if ($request->hasFile('images')) {
                $this->uploadMedia($request, auth()->user());
                                $admin = User::where('user_type', 'admin')->get();
                //dd();
                event(new PostActionNotification(
                    $admin->pluck('id')->toArray(),
                    [User::class],
                    [
                        'title' => 'Account review request from ' . auth()->user()->full_name,
                        'body' => 'Account review request from ' . auth()->user()->full_name,
                        'object_id' => 0,
                        'actor_id' => auth()->user()->id,
                        'actor_type' => 'App\Models\User',
                        'type' => 'account-review-request',
                        'object' => 'account-review-request',

                    ],
                    null,
                    true
                ));

            }
        }
        $user = tap(User::find(auth()->user()->id))->update($attributes);
        return Helper::response(["user" => new UserResource($user, $request->bearerToken())], 200);
    }

    /**
     * Find user by id
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function getUserDetails($id)
    {
        $data = User::with(['preferences', 'categories', 'schedule'])->findOrFail( $this->decode($id) );

        return $this->response(["user" => new UserResource($data)], 200);
    }

    public function me(Request $request){
        $data = User::with(['preferences', 'categories', 'schedule'])->findOrFail( auth()->user()->id );
        return $this->response(["user" => new UserResource($data)], 200);
    }

    /**
     * get auth user profile
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function getAuthUserDetails()
    {
        $data = User::with(['preferences', 'categories', 'schedule'])->findOrFail( auth()->user()->id );
        return $this->response(["user" => new UserResource($data)], 200);
    }

    public function profileSetup(Request $request)
    {
        $attributes                     = $request->all();
        if (auth()->user()->user_role == 'nanny') {
            if ($request->hasFile('images')) {
                $this->uploadMedia($request, auth()->user());
                $admin = User::where('user_type', 'admin')->get();
                event(new PostActionNotification(
                    $admin->pluck('id')->toArray(),
                    [User::class],
                    [
                        'title' => 'Account review request from ' . auth()->user()->full_name,
                        'body' => 'Account review request from ' . auth()->user()->full_name,
                        'object_id' => 0,
                        'actor_id' => auth()->user()->id,
                        'actor_type' => 'App\Models\User',
                        'type' => 'account-review-request',
                        'object' => 'account-review-request',

                    ],
                    null,
                    true
                ));
            }
            auth()->user()->preferences()->sync($this->decodeIds($request->preferences));
            auth()->user()->categories()->sync($this->decodeIds($request->categories));
        }
        $user   = tap(User::find(auth()->user()->id))->update($attributes);
        return Helper::response(["user" => new UserResource($user, $request->bearerToken())], 200);
    }

    public function profileSetupNanny(UserUpdateRequest $request)
    {
        $checkr                         = new Checkr();
        $attributes                     = $request->all();
        $candidate_id                   = auth()->user()->candidate_id;

        try{
            if(auth()->user()->candidate_id == null){
                $httpRequest                                    = $this->httpUser($request);
                $result                                         = $checkr->createCandidate($httpRequest);
                if(!isset($result->error)){
                    $candidate_id                                   = $result->id;
                    $attributes['candidate_id']                     = $candidate_id;
                }else{
                    return Helper::response(null, 422, false, $result);
                }
            }
            $report                                             = $checkr->createInvitation($candidate_id, $request);
            if(isset($report->error)){
                return Helper::response(null, 422, false, $report->error);
            }
        }catch(Exception $e){
            return Helper::response(null, 422, false, [ json_encode($e->getMessage()) ]);
        }
        if (auth()->user()->user_role == 'nanny') {
            auth()->user()->verification_status = 'pending';
            auth()->user()->save();
            $admin = User::where('user_type', 'admin')->get();
            event(new PostActionNotification(
                $admin->pluck('id')->toArray(),
                [User::class],
                [
                    'title' => 'Account review request from ' . auth()->user()->full_name,
                    'body' => 'Account review request from ' . auth()->user()->full_name,
                    'object_id' => 0,
                    'actor_id' => auth()->user()->id,
                    'actor_type' => 'App\Models\User',
                    'type' => 'account-review-request',
                    'object' => 'account-review-request',

                ],
                null,
                true
            ));

            auth()->user()->preferences()->sync($this->decodeIds($request->preferences));
            auth()->user()->categories()->sync($this->decodeIds($request->categories));
        }
        $user = tap(User::find(auth()->user()->id))->update($attributes);
        return Helper::response(["user" => new UserResource($user, $request->bearerToken())], 200);
    }

    public function mediaVerification(Request $request)
    {
        $attributes = $request->all();
        if (auth()->user()->user_role == 'nanny') {
            if ($request->hasFile('images')) {
                $this->uploadMedia($request, auth()->user());
                auth()->user()->verification_status = 'pending';
                auth()->user()->save();
                  $admin = User::where('user_type', 'admin')->get();
                //dd();
                event(new PostActionNotification(
                    $admin->pluck('id')->toArray(),
                    [User::class],
                    [
                        'title' => 'Account review request from ' . auth()->user()->full_name,
                        'body' => 'Account review request from ' . auth()->user()->full_name,
                        'object_id' => 0,
                        'actor_id' => auth()->user()->id,
                        'actor_type' => 'App\Models\User',
                        'type' => 'account-review-request',
                        'object' => 'account-review-request',

                    ],
                    null,
                    true
                ));

            }

        }
        $user = tap(User::find(auth()->user()->id))->update($attributes);
        return Helper::response(["user" => new UserResource($user, $request->bearerToken())], 200);
    }

    public function deleteAccount()
    {
        $user = User::find(auth()->user()->id);
        $user->delete();
        return Helper::response([], 200, [__('Your account has been deleted successfully')]);
    }


    public function updatePicture(Request $request)
    {
        if ($request->file('profile_image')) {
            $extension = $request->file('profile_image')->getClientOriginalExtension();
            $path = $request->file('profile_image')->storeAs(
                'avatar',
                Str::random(10) . '.' . $extension,
                env('DISK')
            );
            $media = auth()->user()->update([
                'profile_image' => $path,
            ]);
        }
        $user = auth()->user()->fresh();
        return Helper::response(["user" => new UserResource($user)], 200, false, [__('profile image has been updated successfully')]);
    }

    private function uploadMedia($request, $user)
    {
        if ($request->images !== null) {

            foreach ($request->images as $key => $medium) {
                if (is_file($medium)) {
                    $user->userMedia()->where('media_title', $key)->delete();

                    $file = \Storage::disk(env('DISK'))->put('user', $medium);

                    $user->userMedia()->create([
                        'media_type' => $medium->getClientMimeType(),
                        'media_url' => $file,
                        'media_title' => $key
                    ]);
                }
            }
        }
    }

    public function addDevice(Request $request)
    {
        if ($request->has('udid')) {
            auth()->user()->userDevice($request, auth()->user()->access_token);
        }
        return Helper::response([], 200, null, [__('Device added successfully')]);
    }

    public function deleteDevice(Request $request)
    {
        if ($request->has('udid')) {
            auth()->user()->devices()->where('udid', $request->udid)->delete();
        }
        return Helper::response([], 200, null, [__('Device deleted successfully')]);
    }

    public function webhook(Request $request){
        $requestData = $request->all();
        \Log::info('----------Start Webhook----------');
        \Log::info($requestData['data']['object']);
        \Log::info($request->data['object']['candidate_id']);
        \Log::info('----------End Webhook----------');
        if (isset($requestData['data']) && is_array($requestData['data'])) {
            if (isset($requestData['data']['object']) && is_array($requestData['data']['object'])) {
                if (isset($requestData['data']['object']['candidate_id'])) {
                    $candidate_id = $requestData['data']['object']['candidate_id'];
                    $user = User::where('candidate_id', $candidate_id)->first();
                    if($user){
                        $user->update([
                            'is_approved' => 1,
                            'verification_status' => 'approved'
                        ]);
                    }
                    // if ($request->data['object']['status'] == 'complete') {
                    //     \Log::info($user);
                    // }
                }
            }
        }
        \Log::info('--------- Request End Webhook----------');
        Helper::response(null, 200);
    }

    private function httpUser($request){
        $user = [
            'email'                 => auth()->user()->email ?? false,
            'work_locations'        => $request->work_locations  ?? false
        ];
        return $user;
    }


    private function attributes($request)
    {
        return $request->only([
            'first_name',
            'last_name',
            'full_name',
            'phone',
            'gender',
            'dob',
            'user_type',
        ]);
    }

}
