<?php

namespace App\Http\Controllers;

use App\Events\PostActionNotification;
use App\Models\TaxAndCharge;
use App\Models\Appointment;
use App\Models\User;
use App\Traits\HashId;
use App\Traits\Response;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class WalletController extends Controller
{
    use HashId, Response;
    /**
     * index
     *
     * @return array
     */


    public function index(){
        $wallet = auth()->user()->wallet()->first();
        $tax_and_charges = TaxAndCharge::where('charged_for','nanny')->get();
          $charges = $this->calculations($wallet->amount);
            $total = $wallet->amount - $charges['tax_amount'] - $charges['charge_amount'] - $charges['fee_amount'];
            $wallet->amount = $total;
        return $this->response([
            "tax_and_charges" => $tax_and_charges,
            "wallet" =>$wallet,
            "wallet_day" => $this->walletToday(),
            "wallet_week" => $this->walletWeek(),
            "wallet_month" => $this->walletMonth()
        ],200,false,["wallet details"]);

    }

    /**
     * withDraw
     *
     * @param  mixed $request
     * @return void
     */
    public function withDraw(Request $request)
    {

        try {

            $charges = $this->calculations($request->amount);
            $total = $request->amount + $charges['tax_amount'] + $charges['charge_amount'] + $charges['fee_amount'];
            $wallet = auth()->user()->wallet->withdraw($total, 'withdraw');
            $admin = User::where('user_type', 'admin')->get();

            event(new PostActionNotification(
                $admin->pluck('id')->toArray(),
                ['App\Models\User'],
                [
                    'title' => 'Withdraw amount request from ' . auth()->user()->full_name,
                    'body' => 'Withdraw amount request from ' . auth()->user()->full_name,
                    'object_id' => 0,
                    'actor_id' => auth()->user()->id,
                    'actor_type' => 'App\Models\User',
                    'type' => 'withdraw-amount-request',
                    'object' => 'withdraw amount request',

                ],
                null,
                true
            ));

            event(new PostActionNotification(
                [auth()->user()->id],
                ['App\Models\User'],
                [
                    'title' => 'Amount Withdrawn',
                    'body' => $total . '$ has been withdrawn successfully',
                    'object_id' => 0,
                    'actor_id' => 1,
                    'actor_type' => 'App\Models\User',
                    'type' => 'withdraw-amount-request',
                    'object' => 'withdraw amount request',

                ],
                null,
                true
            ));

            return $this->response(
                [
                    'transaction' => $wallet
                ],
                200,
                false,
                ['transaction completed successfully']
            );
        } catch (\Exception $e) {
            // dd($e);
            return $this->response("", 422, false, [$e->getMessage()]);
        }
    }



    /**
     * walletToday
     *
     * @return return array of wallet and calculations
     */
    private function walletToday(){

        $wallet_today = auth()->user()->wallet()->with('transactions', function($q){
            $q->whereDate('created_at',Carbon::now())->get();
            $q->where('amount_type','payment');
        })->first();
        $Appointment = Appointment::where('user_id',auth()->user()->id)->whereIn('booking_type',['instant','selection'])
        ->whereDate('created_at',Carbon::now())->get();
        $calculations = $this->calculations($wallet_today->transactions->sum('amount'));


        // return [
        //     "rides_count"=>$wallet_today->transactions->count(),
        //     "earned_amount"=>$wallet_today->transactions->sum('amount'),
        //     "tax_amount" => $calculations['tax_amount'],
        //     "charge_amount" => $calculations['charge_amount'],
        //     "fee_amount" => $calculations['fee_amount'],
        //     "total_amount" => $wallet_today->transactions->sum('amount') - $calculations['tax_amount'] - $calculations['charge_amount'] - $calculations['fee_amount'],
        //       ];
        
          return [
            "number_of_jobs"=>$wallet_today->transactions->count(),
            "total_earnings"=>$wallet_today->transactions->sum('amount'),
            "hourly_rate"=>auth()->user()->service_charge,
            "total_hours_worked"=>$Appointment->sum('working_hours'),
            "tax_amount" => $calculations['tax_amount'],
            "charge_amount" => $calculations['charge_amount'],
            "fee_amount" => $calculations['fee_amount'],
            "total_amount" => $wallet_today->transactions->sum('amount') - $calculations['tax_amount'] - $calculations['charge_amount'] - $calculations['fee_amount'],
               ];


    }


    /**
     * walletWeek
     *
     * @return return array of wallet and calculations
     */
    private function walletWeek(){

        $wallet_week = auth()->user()->wallet()->with('transactions', function($q){
            $q->whereBetween(DB::raw('DATE(created_at)'), array(Carbon::now()->startOfWeek(Carbon::MONDAY)->format('y-m-d'), Carbon::now()->endOfWeek(Carbon::SUNDAY)->format('y-m-d')))->get();
            $q->where('amount_type','payment');

        })->first();
         $Appointment = Appointment::where('user_id',auth()->user()->id)->whereIn('booking_type',['instant','selection'])
        ->whereBetween(DB::raw('DATE(created_at)'), array(Carbon::now()->startOfWeek(Carbon::MONDAY)->format('y-m-d'), Carbon::now()->endOfWeek(Carbon::SUNDAY)->format('y-m-d')))->get();
        $calculations = $this->calculations($wallet_week->transactions->sum('amount'));


       return [
            "number_of_jobs"=>$wallet_week->transactions->count(),
            "total_earnings"=>$wallet_week->transactions->sum('amount'),
            "hourly_rate"=>auth()->user()->service_charge,
            "total_hours_worked"=>$Appointment->sum('working_hours'),
            "tax_amount" => $calculations['tax_amount'],
            "charge_amount" => $calculations['charge_amount'],
            "fee_amount" => $calculations['fee_amount'],
            "total_amount" => $wallet_week->transactions->sum('amount') - $calculations['tax_amount'] - $calculations['charge_amount'] - $calculations['fee_amount'],
               ];



    }


    /**
     * walletMonth
     *
     * @return return array of wallet and calculations
     */
    private function walletMonth(){


        $wallet_month = auth()->user()->wallet()->with('transactions', function($q){
            $q->whereBetween(DB::raw('DATE(created_at)'), array(Carbon::now()->startOfMonth()->format('y-m-d'), Carbon::now()->endOfMonth()->format('y-m-d')))->get();
            $q->where('amount_type','payment');

        })->first();
         $Appointment = Appointment::where('user_id',auth()->user()->id)->whereIn('booking_type',['instant','selection'])
        ->whereBetween(DB::raw('DATE(created_at)'), array(Carbon::now()->startOfMonth()->format('y-m-d'), Carbon::now()->endOfMonth()->format('y-m-d')))->get();
        $calculations = $this->calculations($wallet_month->transactions->sum('amount'));

        return [
            "number_of jobs"=>$wallet_month->transactions->count(),
            "total_earnings"=>$wallet_month->transactions->sum('amount'),
            "hourly_rate"=>auth()->user()->service_charge,
            "total_hours_worked"=>$Appointment->sum('working_hours'),
            "tax_amount" => $calculations['tax_amount'],
            "charge_amount" => $calculations['charge_amount'],
            "fee_amount" => $calculations['fee_amount'],
            "total_amount" => $wallet_month->transactions->sum('amount') - $calculations['tax_amount'] - $calculations['charge_amount'] - $calculations['fee_amount'],
               ];



    }




    //useful function


    /**
     * calculations
     *
     * @param  mixed $wallet
     * integer value (amount)
     * @return void
     */
    private function calculations($wallet){
        $tax_and_charges = TaxAndCharge::where('charged_for','nanny')->get();
        $sub_amount = $wallet;
        $tax_amount[] = null;
        $charge_amount[] = null;
        $fee_amount[] = null;
        foreach ($tax_and_charges as $tax) {
            if ($tax->operation_type == 'percent' && $tax->charge_type == 'tax') {
                $tax_amount[] = $sub_amount * $tax->value / 100;
            } elseif ($tax->operation_type == 'amount' && $tax->charge_type == 'tax') {
                $tax_amount[] = $tax->value;
            } elseif ($tax->operation_type == 'percent' && $tax->charge_type == 'charge') {
                $charge_amount[] = $sub_amount * $tax->value / 100;
            } elseif ($tax->operation_type == 'amount' && $tax->charge_type == 'charge') {
                $charge_amount[] = $tax->value;
            }elseif ($tax->operation_type == 'percent' && $tax->charge_type == 'fee') {
                $charge_amount[] = $sub_amount * $tax->value / 100;
            } elseif ($tax->operation_type == 'amount' && $tax->charge_type == 'fee') {
                $charge_amount[] = $tax->value;
            }
        }

         $data=
         [
            "tax_amount" => array_sum($tax_amount),
            "charge_amount"=> array_sum($charge_amount),
            "fee_amount"=> array_sum($fee_amount)
        ];

        return $data;



    }

}

