<?php

namespace App\Http\Resources;

use App\Helpers\Helper;
use Illuminate\Http\Resources\Json\JsonResource;

class UserResource extends JsonResource
{
    /**
     * @var mixed|null $accessToken
     */
    private $accessToken;

    /**
     * UserResource constructor.
     * @param $resource
     * @param null $accessToken
     */
    public function __construct($resource, $accessToken = null)
    {
        parent::__construct($resource);
        $this->accessToken = $accessToken;
        $this->resource = $resource;
    }

    /**
     * Transform the resource into an array.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return array|\Illuminate\Contracts\Support\Arrayable|\JsonSerializable
     */
    public function toArray($request)
    {
        $response = parent::toArray($request);

        $response['id'] = Helper::encode($this->id);
        $response['token'] = $this->accessToken;
        $response['token_type'] = 'bearer';
        $response['issue_date'] = $this->issue_date !== null ? $this->issue_date : null;
        if($this->user_role == 'nanny'){
            $response['preferences'] = PreferencesResource::collection($this->preferences);
            $response['categories'] = CategoriesResource::collection($this->categories);
            $response['nanny_media'] = $this->userMedia()->count() > 0 ? $this->userMedia : null;
        }
        $response['schedule'] = $this->schedule;
        unset($response['deleted_at']);

        return $response;
    }
}
