<?php

namespace App\Jobs;

use App\Models\User;
use Illuminate\Auth\Events\Registered;
use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldBeUnique;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Queue\SerializesModels;
use Twilio\Rest\Client;



class ProcessVerification implements ShouldQueue
{
    use Dispatchable, InteractsWithQueue, Queueable, SerializesModels;

    /**
     * @var $user
     */
    private $user;

    /**
     * Twilio account ID
     */
    public $account_sid;

    /**
     * Twilio auth token
     */
    public $auth_token;

    /**
     * Twilio phone number
     */
    public $twilio_number;


    /**
     * Create a new job instance.
     *
     * @return void
     */
    public function __construct(User $user)
    {
        $this->user = $user;
        $this->account_sid = env('TWILIO_ACCOUNT_SID');
        $this->auth_token = env('TWILIO_AUTH_TOKEN');
        $this->twilio_number = env('TWILIO_NUMBER');
    }

    /**
     * Execute the job.
     *
     * @return void
     */
    public function handle()
    {
        try {
            $client = new Client($this->account_sid, $this->auth_token);

            $message = "Hi, " . $this->user->first_name . "\n\n" . "(" . $this->user->forgot_password_code . ") " .
                "is your 4-digits confirmation code for changing password of " . "\n\n" . "Nannies on wheels.";

            $client->messages->create(
                $this->user->phone,
                ['from' => $this->twilio_number, 'body' => $message]
            );
        } catch (\Exception $e) {
            return false;
        }
    }
}
