<?php

namespace App\Listeners;

use App\Models\Activity;
use App\Models\UserDevice;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Support\Facades\Log;
use LaravelFCM\Facades\FCM;
use LaravelFCM\Message\OptionsBuilder;
use LaravelFCM\Message\PayloadDataBuilder;
use LaravelFCM\Message\PayloadNotificationBuilder;

class SendActionNotification
{
    /**
     * Create the event listener.
     *
     * @return void
     */
    public function __construct()
    {
        //
    }

    /**
     * Handle the event.
     *
     * @param  object  $event
     * @return void
     */
    public function handle($event)
    {

        try {
            $user_ids = $event->ids;
            $user_types = $event->user_types;
            $payload = $event->payload;
            $type = $event->type;
            $devices = UserDevice::whereIn('user_id', $user_ids)->where('is_notify', 1)
                ->get();

            // dd($devices->toArray());

            // NOTIFICATION INSERT
            if ($event->activity) {
                foreach ($user_ids as $key => $user_id) {
                    $payload['user_id'] = $user_id;
                    //$payload['user_type'] = $user_types[$key];
                    Activity::create($payload);
                }
            }

            /*         $user = auth('api')->user();
            // dd($user);
            $user = $user->fresh();
 */
            if ($devices->count()) {
                //$devices = $devices->toArray();
                $android_dev = $devices->where('device_type', 'android')->pluck(['device_token'])->toArray();
                $ios_dev = $devices->where('device_type', 'ios')->pluck(['device_token'])->toArray();
                $all_dev = $devices->pluck(['device_token'])->toArray();

                // dd($devices->toArray());

                $optionBuilder = new OptionsBuilder();
                $optionBuilder->setPriority('high');
                $optionBuilder->setTimeToLive(60 * 20);

                $notificationBuilder = new PayloadNotificationBuilder(@$payload['title']);
                $notificationBuilder->setBody(@$payload['body'])
                    ->setSound('default');

                $dataBuilder = new PayloadDataBuilder();
                $dataBuilder->addData(['custom' => $payload]);

                $option = $optionBuilder->build();
                $notification = $notificationBuilder->build();
                $data = $dataBuilder->build();

                /*       if (sizeof($android_dev))
                    $downstreamResponse = FCM::sendTo($android_dev, $option, null, $data);

                if (sizeof($ios_dev))
                    $downstreamResponse = FCM::sendTo($ios_dev, $option, $notification, $data); */
                    //dd($payload['type']);
                    if($payload['type'] == 'incoming_call'){
                        // $downstreamResponse = FCM::sendTo($android_dev, $option, $notification, $data);

                        $silentNotificationBuilder = new PayloadNotificationBuilder();
                        $silentNotificationBuilder->setSound(false);
                        $silentNotificationBuilder = $silentNotificationBuilder->build();
                        $downstreamResponse = FCM::sendTo($android_dev, $option, $notification, $data);
                    }
                    elseif($payload['type'] == 'incoming_call_end')
                    {
                        $silentNotificationBuilder = new PayloadNotificationBuilder();
                        $silentNotificationBuilder->setSound(false);
                        $silentNotificationBuilder = $silentNotificationBuilder->build();
                        $downstreamResponse = FCM::sendTo($all_dev, $option, $notification, $data);
                    }
                    else{
                        $downstreamResponse = FCM::sendTo($all_dev, $option, $notification, $data);
                    }

                $logged = "\n
                    SUCCESS: {$downstreamResponse->numberSuccess()} | FAILED: {$downstreamResponse->numberFailure()}";

                Log::debug($logged);
                Log::info($downstreamResponse->tokensWithError());
                Log::debug($logged);
                log::info($data);
                log::info($notification);
            }
        } catch (\Exception $e) {
            Log::debug($e);
            return false;
        }
    }
}
