<?php

namespace App\Listeners;

use App\Events\PostActionNotification;
use App\Models\Activity;
use App\Models\UserDevice;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Support\Facades\Log;
use LaravelFCM\Facades\FCM;
use LaravelFCM\Message\OptionsBuilder;
use LaravelFCM\Message\PayloadDataBuilder;
use LaravelFCM\Message\PayloadNotificationBuilder;
use App\Http\Helpers\Helper;

use App\Http\Resources\UserResource;

class SendActionNotification
{
    /**
     * Create the event listener.
     *
     * @return void
     */
    public function __construct()
    {
        //
    }

    /**
     * Handle the event.
     *
     * @param $event
     * @return mixed
     */
    public function handle($event)
    {
        try {
            $user_ids = $event->ids;
            $payload = $event->payload;
            $devices = UserDevice::whereIn('user_id', $user_ids)->where('notifications', 1)->get();

            //$actor_id = @auth('api')->user()->first(['id', 'full_name', 'image'])->toArray();


            // NOTIFICATION INSERT
            if ($event->activity) {
                foreach ($user_ids as $key => $user_id) {

                    $user = is_array($user_id) ? $user_id[0] : $user_id;

                    $activity = $payload;
                    if (@$payload['cron']) {
                        $activity['actor_id'] = $payload['actor_id'];
                    } else {
                        $activity['actor_id'] = @auth('api')->user()->id ? auth('api')->user()->id : auth('web')->user()->id;
                    }
                    $activity['user_id'] = is_string($user) ? Helper::hashidDecode($user) : $user;
                    $activity['object_id'] = is_string($payload['object_id']) ? Helper::hashidDecode($payload['object_id']) : $payload['object_id'];
                    Activity::create($activity);
                }
            }

            //$payload['actor'] = new UserResource( @auth('api')->user() );

            if ($devices->count()) {
                //$devices->pluck(['device_token']);

                //$devices = $devices->toArray();
                $android_dev = $devices->where('device_type', 'android')->pluck(['device_token'])->toArray();
                $ios_dev = $devices->where('device_type', 'ios')->pluck(['device_token'])->toArray();

                $optionBuilder = new OptionsBuilder();
                $optionBuilder->setTimeToLive(60 * 20);

                $notificationBuilder = new PayloadNotificationBuilder(@$payload['title']);
                $notificationBuilder->setBody(@$payload['body'])
                    ->setSound('default');

                $dataBuilder = new PayloadDataBuilder();
                $dataBuilder->addData(['custom' => $payload]);

                $option = $optionBuilder->build();
                $notification = $notificationBuilder->build();
                $data = $dataBuilder->build();

                if (sizeof($android_dev))
                    $downstreamResponse = FCM::sendTo($android_dev, $option, null, $data);

                if (sizeof($ios_dev))
                    $downstreamResponse = FCM::sendTo($ios_dev, $option, $notification, $data);


                $logged = "\n
                SUCCESS: {$downstreamResponse->numberSuccess()} | FAILED: {$downstreamResponse->numberFailure()}";

                Log::debug($logged);
                Log::info($downstreamResponse->tokensWithError());
                Log::debug($logged);
                log::info($data);
                log::info($notification);
            }
        } catch (\Exception $e) {
            Log::debug($e);
            //dd( $e );
            return false;
        }
    }
}
