<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;

class Appointment extends Model
{
    use HasFactory, SoftDeletes;

    protected $fillable = [
        'booking_type',
        'job_title',
        'qualification',
        'service_category_id',
        'job_type',
        'start_date',
        'pay_type',
        'working_hours',
        'pay_range',
        'ethnicity',
        'language',
        'address',
        'latitude',
        'longitude',
        'description',
        'preferences',
        'user_id',
        'booking_id',
        'booking_type',
        'job_title',
        'qualification',
        'service_category_id',
        'service_provider_id',
        'ethnicity',
        'language',
        'address',
        'latitude',
        'longitude',
        'addressline2',
        'duration',
        'start_date',
        'start_time',
        'end_date',
        'end_time',
        'additional_notes',
        'appointment_cancel_time',
        'appointment_expire_time',
        'appointment_type',
        'pay_type',
        'status',
        'image',
        'started_at',
        'service_status',
        'description',
        'transaction_reference',
        'verification_code',
        'parent_id',
        'rate_per_hour',
        'schedule',
    ];

        /**
     * The booted method of the model
     */
    protected static function booted()
    {
        static::creating(function ($appointment) {
            $appointment->verification_code = mt_rand(1000, 9999);
        });
    }

    protected $appends = [
        'image_url',
        'apply_status'
    ];

    protected $casts  = [
        'schedule' => 'array',
    ];

    public function setImageAttribute($image)
    {
        if ($image) {
            $this->attributes['image'] = $image->store("proofs", 'public');
        }
    }

    public function getImageUrlAttribute()
    {
        return $this->image ? asset("storage/$this->image") : null;
    }

    public function getApplyStatusAttribute()
    {
        return $this->apply()->where('user_id', auth()->user()->id)->first() != null ? $this->apply->status : null;
    }

    public function customer()
    {
        return $this->belongsTo(User::class, 'user_id');
    }

    public function service_provider()
    {
        return $this->belongsTo(User::class, 'service_provider_id');
    }

    public function review()
    {
        return $this->hasOne(Rating::class, 'appointment_id');
    }

    public function service()
    {
        return $this->hasOne(Rating::class, 'appointment_id');
    }

    public function appliers()
    {
        return $this->belongsToMany(User::class, 'job_appliers', 'appointment_id', 'user_id');
        // ->select(['users.id', 'franchise_id', 'name', 'email', 'profile_image']);
    }
    public function apply()
    {
        return $this->hasOne(JobApplier::class);
        // ->select(['users.id', 'franchise_id', 'name', 'email', 'profile_image']);
    }

    public function refund()
    {

        if ($this->invoice_id) {
            $invoice = $this->invoice;

            if (!$invoice->paid_at) return;

            $amount_per_day = $invoice['total_amount'] / $invoice['days'];
            $transaction = $this->customer->wallet->deposit($amount_per_day, 'refund', $this->id);

            $body = "£ $amount_per_day is refunded to your wallet of $this->booking_id.";
            event(new SendAction([
                $this->user_id,
            ], [
                'actor_id' => 0,
                'action' => config('constants.BOOKING_REFUNDED'),
                'object' => 'transaction',
                'object_id' => $transaction->id,
                'content' => $body
            ]));
        }
    }

    public function disburse()
    {

        if ($this->invoice_id) {
            $invoice = $this->invoice;
            $amount_per_day = $invoice['total_amount'] / $invoice['days'];

            $admin_percent = PaymentSetting::where('type', 'admin_percent')->first()['amount'];

            $admin_share = round($amount_per_day * ($admin_percent / 100), 2);
            $provider_share = $amount_per_day - $admin_share;

            $transaction = $this->service_provider->wallet->deposit($provider_share, 'payment', $this->id);
            $this->service_provider->franchise->wallet->deposit($admin_share, 'payment', $this->id);

            $body = "£ $provider_share is added to your wallet of $this->booking_id.";
            event(new SendAction([
                $this->service_provider_id,
            ], [
                'actor_id' => 0,
                'action' => config('constants.CREDITED'),
                'object' => 'transaction',
                'object_id' => $transaction->id,
                'content' => $body
            ]));
        }
    }

    public function category()
    {
        return $this->belongsTo(Category::class, 'service_category_id');
    }

    public function dispute()
    {
        return $this->hasOne(AppointmentDispute::class);
    }
}
