<?php
namespace App\Services;

use GuzzleHttp\Client;

class checkr
{
    private $domain = '';
    private $api_key = '';
    private $package = '';
    // tasker_standard
    public function __construct($api_version = 'v1', $package = 'basicPackage')
    {
        $this->domain       = "https://api.checkr-staging.com/v1";
        $this->api_key      = "e1b0b6695fb55e966875090356182343fdb8de1a";
        $this->api_version  = $api_version;
		$this->package      = $package;
    }
    /**
	 * Creates a new candidate
	 * @param  array  $candidate_params  name, address, and sensitive information i.e ssn, drivers license #, drivers state
	 * @return object $candidate         object returned from resource
	 */
	public function createCandidate($candidate_params = []) {
	    $url = "{$this->domain}/candidates";
		$candidate = $this->request($candidate_params, $url, true);
		return $candidate;
	}
    
    /**
	 * Create background check report
	 * @param  string $candidate_id  candidate's id
	 * @return object                a report object
	 */
	public function createInvitation($candidate_id, $request) {
        $url = "{$this->domain}/invitations";
		$report_params = [
		    "candidate_id"      => $candidate_id,
		    "package"           => $this->package,
		    "work_locations"    => $request->work_locations
		    //  "work_locations"    => [
            //             [
            //                 "state"     => "CA",
            //                 "country"   => "US",
            //                 "city"      => "San Francisco"
            //             ]
            //         ]
		];
		return $this->request($report_params, $url, true);
	}
	
    /**
	 * Create background check report
	 * @param  string $candidate_id  candidate's id
	 * @return object                a report object
	 */
	public function createReport($candidate_id) {
        $url = "{$this->domain}/reports";
		$report_params = array(
		    "candidate_id"              => $candidate_id,
		    "package"                   => $this->package
		);
		return $this->request($report_params, $url, true);
	}

	/**
	 * Retrieves a list of candidates
	 * @return array  an array of candidates
	 */
	public function listCandidates() {
        $url = "{$this->domain}/candidates/?include=candidate,ssn_trace,county_criminal_searches";
		return $this->request([], false);

	}
	
	/**
	 * Retrieves a list of candidates
	 * @return array  an array of candidates
	 */
	public function listPackages() {
        $url = "{$this->domain}/nodes?include=packages";
		return $this->request([], false);

	}

	/**
	 * Retrieve a single candidate
	 * @param  string $id  candidate's id
	 * @return object      a candidate object
	 */
	public function getCandidate($id) {
        $url = "{$this->domain}/candidates/".$id."?include=candidate,ssn_trace,county_criminal_searches";
		return $this->request([], false);

	}

	/**
	 * Retrieve a single candidate report
	 * @param  string $report_id  id of report
	 * @return object             a report object
	 */
	public function getReport($report_id) {
        $url = "{$this->domain}/reports/".$report_id."?include=candidate,ssn_trace,county_criminal_searches";
		return $this->request([], false);

	}

	/**
	 * List all reports
	 * @return array  an array of report objects
	 */
	public function listReports() {

		foreach ($this->listCandidates() as $candidate) {

			foreach ($candidate->report_ids as $report_id) {

				$reports[] = $this->getReport($report_id);
			}
		}

		return $reports;

	}

	/**
	 * Sends curl request to get data
	 * @param  array   $params   array of params for checkr
	 * @param  string  $url      checkr api
	 * @return json            	 returns json object
	 */
	private function request($params = [], $url, $post = true) {

		$curl = curl_init();
		curl_setopt($curl, CURLOPT_URL, $url);
		curl_setopt($curl, CURLOPT_USERPWD, $this->api_key . ":");
		curl_setopt($curl, CURLOPT_HTTPHEADER, array('Content-Type: application/json'));
		curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($curl, CURLOPT_POST, $post);
		curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, 5);

		if (count($params))
		    curl_setopt($curl, CURLOPT_POSTFIELDS, json_encode($params));
            // 			curl_setopt($curl, CURLOPT_POSTFIELDS, http_build_query($params));


		$json = curl_exec($curl);
		$http_status = curl_getinfo($curl, CURLINFO_HTTP_CODE);

		curl_close($curl);

		$response = json_decode($json);

		return $response;

	}
}
