<?php

use App\Http\Controllers\ActivityController;
use App\Http\Controllers\AppointmentController;
use App\Http\Controllers\AppointmentDisputeController;
use App\Http\Controllers\AuthenticationController;
use App\Http\Controllers\BlockUserController;
use App\Http\Controllers\ChangePasswordController;
use App\Http\Controllers\ContentController;
use App\Http\Controllers\FavoriteUserController;
use App\Http\Controllers\HomeController;
use App\Http\Controllers\JobController;
use App\Http\Controllers\MessageController;
use App\Http\Controllers\PackageController;
use App\Http\Controllers\PasswordResetController;
use App\Http\Controllers\PreferenceController;
use App\Http\Controllers\RatingController;
use App\Http\Controllers\RegistrationController;
use App\Http\Controllers\UserController;
use App\Http\Controllers\SubscriptionController;
use App\Http\Controllers\TimeSlotsController;
use App\Http\Controllers\UserCardController;
use App\Http\Controllers\UserPreferencesController;
use App\Http\Controllers\UserScheduleController;
use App\Http\Controllers\VideoCallController;
use App\Http\Controllers\WalletController;
use App\Http\Controllers\LocationController;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Route;

/*
|--------------------------------------------------------------------------
| API Routes
|--------------------------------------------------------------------------
|
| Here is where you can register API routes for your application. These
| routes are loaded by the RouteServiceProvider within a group which
| is assigned the "api" middleware group. Enjoy building your API!
|
*/

/* Route::middleware('auth:sanctum')->get('/user', function (Request $request) {
    return $request->user();
}); */


Route::get('/bkLocation', function () {
    \Log::debug('BKLOCATION');
});
\Log::info('--------------start-------------------');
\Log::info(@$_SERVER['HTTP_AUTHORIZATION']);
\Log::info(\Request::fullUrl());
\Log::info('---------------end------------------');
\Log::info('-------------header start--------------------');
\Log::info(\Request::header());
\Log::info(@$_SERVER['REQUEST_METHOD']);
\Log::info('-------------header end--------------------');
\Log::info('-------------json request start--------------------');
\Log::info(json_encode(\Request::all()));
\Log::info('-------------json request end--------------------');
\Log::info('-------------request all start--------------------');
\Log::info(\Request::all());
\Log::info('-------------request all end--------------------');


Route::namespace('App\Http\Controllers')->group(function () {
    Route::get('/info', function () {
        return response()->json([
            "email" => "support@villagenanny.com",
            "phone" => "1234 123 1234",
            "whatsapp" => "1234 123 1234",
            "subscription_display" => false,
            'android' => [
                "version" => "1.0.0",
                "link" => ""
            ],
            'ios' => [
                "version" => "1.0.0",
                "link" => ""
            ],
        ]);
    });

    // Authentication
    Route::post('/login', [AuthenticationController::class, 'login'])->name('login');
    Route::post('/social-login', [AuthenticationController::class, 'socialLogin']);
    Route::post('/guest-login', [AuthenticationController::class, 'guestLogin']);

    // New User
    Route::post('/register', [RegistrationController::class, 'register']);

    // Forgot Password
    Route::post('/forgot-password', [PasswordResetController::class, 'sendResetLink'])
        ->middleware('throttle:6,1');

    Route::post('/forgot-password/{token}', [PasswordResetController::class, 'passwordReset']);

    // Get All Users
    Route::get('/user/all', [UserController::class, 'getUsers']);
    Route::post('/user/webhook', [UserController::class, 'webhook']);

    // password reset
    Route::post('/request-code', [PasswordResetController::class, 'forgotPassword']);
    Route::post('/verify-code', [PasswordResetController::class, 'verifyCode']);
});


// content route
Route::group(
    [
        'middleware' => 'auth:sanctum',
        'prefix' => 'content'
    ],
    function () {
        Route::get('/', [ContentController::class, 'index']);
        Route::get('/{slug}', [ContentController::class, 'show']);
    }
);

// subscription route
Route::group(
    [
        'middleware' => 'auth:sanctum',
        'prefix' => 'subscription'
    ],
    function () {
        Route::post('/subscribe', [SubscriptionController::class, 'subscribe']);
        Route::get('/getSubscription', [SubscriptionController::class, 'getSubscription']);
    }
);

// package route
Route::group(
    [
        'middleware' => 'auth:sanctum',
        'prefix' => 'package'
    ],
    function () {
        Route::get('/', [PackageController::class, 'getPackages']);
        Route::get('/{id}', [PackageController::class, 'getPackage']);
    }
);

// user route
Route::group(
    [
        'middleware' => 'auth:sanctum',
        'prefix' => 'user'
    ],
    function () {
        Route::post('/device', [UserController::class, 'addDevice']);
        Route::delete('/device', [UserController::class, 'deleteDevice']);
        Route::patch('/update', [UserController::class, 'createOrUpdateProfile']);
        Route::patch('/profile-setup', [UserController::class, 'profileSetup']);
        Route::patch('/profile-setup-nanny', [UserController::class, 'profileSetupNanny']);
        Route::patch('/media-verification', [UserController::class, 'mediaVerification']);
        Route::get('/preferences-categories', [PreferenceController::class, 'preferencesCategories']);
        Route::patch('/update-profile-picture', [UserController::class, 'updatePicture']);
        Route::patch('/preferences', [UserPreferencesController::class, 'create']);
        Route::get('/user', [UserController::class, 'getAuthUserDetails']);
        Route::get('/user/{id}', [UserController::class, 'getUserDetails']);
        Route::post('/change-password', [ChangePasswordController::class, 'changePassword']);
        Route::delete('/delete-account', [UserController::class, 'deleteAccount']);
        Route::post('/logout', [AuthenticationController::class, 'logout']);
    }
);

Route::group(
    [
        'middleware' => 'auth:sanctum',
    ],
    function () {
        Route::post('/change-password', [PasswordResetController::class, 'changePassword']);
        Route::get('/rtc/token/{user_id}/{type}', [VideoCallController::class, 'initiate']);
        Route::get('/endCall/{id}', [VideoCallController::class, 'endCall']);
    }
);

Route::group(
    [
        'middleware' => 'auth:sanctum',
        'prefix' => 'location'
    ],
    function () {
        Route::get('/', [LocationController::class, 'getStates']);
    }
);

// wallet route
Route::group(
    [
        'middleware' => 'auth:sanctum',
        'prefix' => 'wallet'
    ],
    function () {
        Route::get('/', [WalletController::class, 'index']);
        Route::post('/withdraw', [WalletController::class, 'withDraw']);
    }
);

// cards route
Route::group(
    [
        'middleware' => 'auth:sanctum',
        'prefix' => 'card'
    ],
    function () {
        Route::get('/', [UserCardController::class, 'showAll']);
        Route::post('/creat', [UserCardController::class, 'create']);
        Route::post('/delete', [UserCardController::class, 'delete']);
        Route::post('/create-card-token', [UserCardController::class, 'createCardToken']);
    }
);

// cards route
Route::group(
    [
        'middleware' => 'auth:sanctum',
        'prefix' => 'schedule'
    ],
    function () {
        Route::post('/create', [UserScheduleController::class, 'update']);
        Route::get('/get/{id}', [UserScheduleController::class, 'view']);
        Route::get('/time-slots', [TimeSlotsController::class, 'all']);
        Route::get('/all-slots', [TimeSlotsController::class, 'allSlots']);
    }
);

// home route
Route::group(
    [
        'middleware' => 'auth:sanctum',
        'prefix' => 'home'
    ],
    function () {
        Route::get('/users', [HomeController::class, 'all']);
        Route::get('/get/{id}', [UserScheduleController::class, 'view']);
        Route::get('/time-slots', [TimeSlotsController::class, 'all']);
        Route::get('/all-slots', [TimeSlotsController::class, 'allSlots']);
        Route::get('/nanny-home', [HomeController::class, 'jobs']);
    }
);

// appointment
Route::group(
    [
        'middleware' => 'auth:sanctum',
        'prefix' => 'appointment'
    ],
    function () {
        Route::post('/create', [AppointmentController::class, 'store']);
        Route::post('/extend-time', [AppointmentController::class, 'extend']);
        Route::get('/', [AppointmentController::class, 'appointments']);
        Route::get('/{id}', [AppointmentController::class, 'appointment']);
        Route::post('/acknowledge', [AppointmentController::class, 'acknowledge']);
        Route::post('/dispute', [AppointmentDisputeController::class, 'create']);

    }
);
// block user
Route::group(
    [
        'middleware' => 'auth:sanctum',
        'prefix' => 'block'
    ],
    function () {
        Route::get('/', [BlockUserController::class, 'index']);
        Route::get('/{id}/add', [BlockUserController::class, 'add']);
        Route::get('/{id}/remove', [BlockUserController::class, 'remove']);
    }
);
// favorite user
Route::group(
    [
        'middleware' => 'auth:sanctum',
        'prefix' => 'favorite'
    ],
    function () {
        Route::get('/', [FavoriteUserController::class, 'index']);
        Route::get('/{id}/add', [FavoriteUserController::class, 'add']);
        Route::get('/{id}/remove', [FavoriteUserController::class, 'remove']);
    }
);

// rating
Route::group(
    [
        'middleware' => 'auth:sanctum',
        'prefix' => 'rating'
    ],
    function () {
        Route::post('/create', [RatingController::class, 'create']);
    }
);

// actiity
Route::group(
    [
        'middleware' => 'auth:sanctum',
        'prefix' => 'activity'
    ],
    function () {
        Route::get('/', [ActivityController::class, 'index']);
    }
);

// jobs
Route::group(
    [
        'middleware' => 'auth:sanctum',
        'prefix' => 'job'
    ],
    function () {
        Route::post('/apply', [JobController::class, 'apply']);
        Route::delete('/apply/{id}/delete', [JobController::class, 'removeApply']);
        Route::get('/appliers/{id}', [JobController::class, 'appliers']);
        Route::post('/hire', [JobController::class, 'hireApplier']);
        Route::post('/reject', [JobController::class, 'rejectApplier']);
        Route::get('/', [JobController::class, 'jobs']);
        Route::get('/{id}', [JobController::class, 'show']);
        Route::delete('/{id}/delete', [JobController::class, 'remove']);
        Route::post('/interested', [JobController::class, 'interested']);
        Route::post('/not-interested', [JobController::class, 'notInterested']);
    }
);

// nanny route
Route::group(
    [
        'middleware' => 'auth:sanctum',
        'prefix' => 'nanny'
    ],
    function () {
        Route::post('/accept-service', [AppointmentController::class, 'acceptService']);
        Route::post('/reject-service', [AppointmentController::class,  'rejectService']);
        Route::post('/arrived', [AppointmentController::class, 'arrived']);
        Route::post('/start-service', [AppointmentController::class, 'startService']);
        Route::post('/end-service', [AppointmentController::class, 'endService']);
    }
);

// message route
Route::group(
    [
        'middleware' => 'auth:sanctum',
        'prefix' => 'message'
    ],
    function () {
        Route::get('/{id}', [MessageController::class, 'show']);
        Route::post('/create', [MessageController::class, 'create']);
    }
);








